"""
General service functions
"""

import logging

from portal import db
from portal.database.models import Component, Course, Group, Project, Role, Submission, User
from portal.database.utils import sql_get_count
from portal.service.errors import ResourceNotFoundError

log = logging.getLogger(__name__)


def update_entity(entity: db.Model, data: dict,
                  allowed: list, write: bool = True):
    """Updates any entity.

    Args:
        entity: Entity that should be updated
        data(dict): Data that should be used to update entity
        allowed(list): List of allowed attributes
        write(bool): if True writes the entity into the database

    Returns: Updated entity
    """
    for param in allowed:
        if hasattr(entity, param) and param in data.keys():
            setattr(entity, param, data[param])

    if write:
        write_entity(entity=entity)
    return entity


def delete_entity(entity: db.Model):
    """Deletes an entity from the database.

    Args:
        entity(db.Model): Database entity instance
    """
    log.debug(f"[DELETE] Entity: {entity}")
    db.session.delete(entity)
    db.session.commit()


def write_entity(entity):
    """Writes an entity to the database.

    Args:
        entity(db.Model): Database entity instance
    """
    log.debug(f"[WRITE] Entity: {entity}")
    db.session.add(entity)
    db.session.commit()


def find_resource(identifier: str, resource: str, query, throws):
    """Gets an instance of the resource

    Args:
        identifier(str): ID or some other identifier of the resource
        resource(str): Resource name
        query: SQLAlchemy query
        throws(bool): True if an exception should be thrown if no instance is found

    Returns: An entity instance or null if none is found

    """
    instance = query.first()

    if not instance:
        log.debug(f"[FIND] Not found {resource.capitalize()}: {identifier}")
        if throws:
            raise ResourceNotFoundError(resource=resource, res_id=identifier)
        else:
            return None
    log.debug(
        f"[FIND] Found {resource.capitalize()} for identifier [{identifier}]: {instance}")
    return instance


def find_group(course: Course, identifier: str, throws=True) -> Group:
    """Gets a Group instance

    Args:
        course(Course): Course instance
        identifier(str): Id of the resource
        throws(bool): Throws an exception if true

    Returns(Group): Group entity instance
    """
    query = Group.query.filter_by(course=course).filter(
        (Group.id == identifier) | (Group.name == identifier)
    )

    return find_resource(
        resource='group',
        identifier=identifier,
        query=query,
        throws=throws
    )


def find_component(identifier: str, throws=True) -> Component:
    """Gets a Component instance

    Args:
        identifier(str): Id of the component
        throws(bool): Throws an exception if true

    Returns(Component): Component entity instance
    """
    query = Component.query.filter(
        (Component.id == identifier) | (Component.name == identifier)
    )
    return find_resource(
        resource='component',
        identifier=identifier,
        query=query,
        throws=throws
    )


def find_course(identifier: str, throws=True) -> Course:
    """Gets a Course instance

    Args:
        identifier(str): Id of the resource
        throws(bool): Throws an exception if true

    Returns(Course): Course entity instance
    """
    query = Course.query.filter(
        ((Course.id == identifier) | (Course.codename == identifier))
    )

    return find_resource(
        resource='course',
        identifier=identifier,
        query=query,
        throws=throws
    )


def find_project(course, identifier: str, throws=True) -> Project:
    """Gets a Project instance

    Args:
        course(Course): Course instance
        identifier(str): Id of the resource
        throws(bool): Throws an exception if true

    Returns(Project): Project entity instance
    """
    query = Project.query.filter_by(course=course).filter(
        (Project.id == identifier) | (Project.name == identifier)
    )

    return find_resource(
        resource='project',
        identifier=identifier,
        query=query,
        throws=throws
    )


def find_role(course: Course, identifier: str, throws=True) -> Role:
    """Gets a Role instance

    Args:
        course(Course): Course instance
        identifier(str): Id of the resource
        throws(bool): Throws an exception if true

    Returns(Role): Role entity instance
    """
    query = Role.query.filter_by(course=course).filter(
        (Role.id == identifier) | (Role.name == identifier)
    )

    return find_resource(
        resource='role',
        identifier=identifier,
        query=query,
        throws=throws
    )


def find_submission(identifier: str, throws=True) -> Submission:
    """Gets a Submission instance

    Args:
        identifier(str): Id of the resource
        throws(bool): Throws an exception if true

    Returns(Submission): Submission entity instance
    """
    return find_resource(
        resource='submission',
        identifier=identifier,
        query=Submission.query.filter_by(id=identifier),
        throws=throws
    )


def find_user(identifier: str, throws=True) -> User:
    """Gets a User instance

    Args:
        identifier(str): Id of the resource
        throws(bool): Throws an exception if true

    Returns(User): User entity instance
    """
    query = User.query.filter(
        ((User.id == identifier) | (User.username ==
                                    identifier) | (User.email == identifier))
    )

    return find_resource(
        resource='user',
        identifier=identifier,
        query=query,
        throws=throws
    )


def get_new_name(source, target: Course, count_func=None) -> str:
    """Gets new name based on the count
    Args:
        source: Source entity
        target(course): Target entity
        count_func: Count func
    Returns(str): New name with suffix

    """

    def __count_general(target, name):
        klass = source.__class__
        query = klass.query.filter_by(course=target).filter(
            klass.name.like(f"%{name}%"))
        return sql_get_count(query)

    count_func = count_func or __count_general
    new_name = source.name
    number = count_func(target=target, name=new_name)
    if number > 0:
        new_name += f"_{number + 1}"
    return new_name
