"""
Notifications service
"""

from emails import Message

from portal.database.models import Course, Group, Role, Submission
from portal.service import emails, general
from portal.service.emails import send_message_to_user


def send_email_notification(context: dict, send_to: dict):
    """Sends an email notification to the user
    Args:
        send_to(dict): Send to object
        context(dict): Notification data
    Returns:
    """
    users = extract_users(**send_to)
    message = emails.get_email_message('notifications', 'new')
    msg = message.create_message()
    for user in users:
        send_message_to_user(user=user, message=msg, context=context)


def send_is_api_notification(data: dict):
    """Writes data to IS MUNI using the IS API
    Args:
        data(dict): Notification data
    Returns: Not implemented
    """
    if data is not None:
        print("Not implemented")
    return None


def extract_send_to(users: list = None, course: Course = None, group: Group = None,
                    role: Role = None) -> dict:
    """Extract send to users
    Args:
        users(list): User id's list
        course(Course): Course instance
        group(Group): Group instance
        role(Role): Role instance

    Returns(dict): Send to dict
    """
    send_to = {}

    if users:
        send_to['users'] = [general.find_user(i) for i in users]
    if course:
        send_to['course'] = general.find_course(course)
    if course and role:
        send_to['role'] = general.find_role(send_to['course'], role)
    if course and group:
        send_to['group'] = general.find_group(send_to['course'], group)

    return send_to


def extract_users(users=None, course: Course = None,
                  group: Group = None, role: Role = None):
    """Extracts users
    Args:
        users(list): List of Users ids
        course(Course): Course instance
        group(Group): Group instance
        role(Role): Role instance

    Returns(list): Extracted users
    """
    users = users or []
    if group and course:
        group_users = course.get_users_by_group(group)
        users.extend(group_users)
    if role and course:
        role_users = course.get_users_by_role(role)
        users.extend(role_users)
    return users


def send_notification(data: dict, notification_type: str):
    """Sends notification based on the notification_type
    Args:
        data(dict): Notification data
        notification_type(str): Notification type
    """
    message = data['data']
    send_to = extract_send_to(**data['send_to'])
    if notification_type == 'email':
        send_email_notification(message, send_to=send_to)
    if notification_type == 'is_api':
        send_is_api_notification(message)


def send_submission_result_message(submission: Submission, data: dict = None):
    """Sends submission result message
    Args:
        submission(Submission): Submission instance
        data(dict): Data
    """
    templates = data['templates']
    context = data['data']
    for template in templates:
        send_submission_template(submission, template, context)


def send_submission_template(
        submission: Submission, template_data: dict, context: dict):
    """Sends submission template
    Args:
        submission(Submission): Submission instance
        template_data(dict): Template data
        context(dict): Context
    """
    user = template_data['user']
    template = emails.create_message('submissions', template_data['template'])

    if user:
        send_message_to_user(message=template, user=user, context=context)
    else:
        send_submissions_message_to_users_based_on_role(
            context, submission, template, template_data
        )


def send_submissions_message_to_users_based_on_role(
        context: dict, submission: Submission, message: Message, template_data: dict):
    """Sends submission message to user based on role
    Args:
        context(dict): Context of the message
        submission(Submission): Submission instance
        message(Message): Message instance
        template_data(dict): Template data
    """
    course: Course = submission.project.course
    project = submission.project
    role = general.find_role(course, template_data['role'])
    author = submission.user
    users = author.get_users_in_group_based_on_role(
        course=course, project=project, role=role)
    for user in users:
        send_message_to_user(message=message, user=user, context=context)
